@@ -605,7 +605,7 @@ strcat
 /*--------------------------------------------------------------------------*/
 
 void
-strncat
+strcatn
 (	char			*d,
 	const char		*s,
 	unsigned long	n
@@ -618,7 +618,7 @@ strncat
 
 	d--;
 
-	while (n-- && *s)
+	while (--n && *s)
 	{
 		*d++ = *s++;
 	}
@@ -1048,6 +1048,69 @@ printf
     return n;
 }
 
+/*--------------------------------------------------------------------------*/
+/* Print quoted string with embedded escapes.
+ */
+
+int
+put_estr
+(	const char	*str
+)
+{	int			quoted = FALSE;
+	int			n      = 0;
+	int			i;
+	int			c;
+
+	if (str[0] == '\"' && str[1])
+	{
+		quoted = TRUE;
+		str++;
+	}
+
+	while ((c = *str++) != '\0')
+	{
+		/* if end of quoted string */
+		if (quoted && c == '\"' && *str == '\0')
+		{
+			break;
+		}
+
+		if (c == '\\')
+		{
+			switch (*str)
+			{
+				case 'n' : c = '\n'; break;
+				case 'r' : c = '\r'; break;
+				case 't' : c = '\t'; break;
+				case 'b' : c = '\b'; break;
+				case 'e' : c = 27;   break;
+				case '\\':           break;
+				default:
+				{
+					if (*str >= '0' && *str <= '7')
+					{
+						i = 3;
+						c = 0;
+
+						/* evaluate upto 3 octal digits */
+						while (i-- && *str >= '0' && *str <= '7')
+						{
+							c = (c * 8) + (*str++ - '0');
+						}
+					}
+					str--;
+					break;
+				}
+			}
+			str++;
+		}
+		put_char(c);
+		n++;
+	}
+
+	return n;
+}
+
 /*--------------------------------------------------------------------------*/
 /* Print unformated string.
  */
@@ -1220,10 +1283,10 @@ equal_strings
 
 char *
 extract
-(	char	*p1,
-	char	*p2,
-	int		len,				/* size of 'p1' buffer						*/
-	int		delim
+(	char		*p1,
+	char		*p2,
+	int			len,				/* size of 'p1' buffer						*/
+	int			delim
 )
 {
 	/* leave room for NULL terminator */
@@ -1297,7 +1360,7 @@ prescan
 		/* first guess at end of line position */
 		eol = dst;
 	
-		/* until end of line */
+		/* until end of data */
 		while (data != end)
 		{
 			/* get next character */
@@ -1398,6 +1461,63 @@ prescan
 
 /*--------------------------------------------------------------------------*/
 
+static struct vmetype {
+	char			*name;
+	unsigned long	code;
+} vmetypes[] = {
+	{"mvme162",	 VME_TYPE_MVME162 },
+	{"mvme166",  VME_TYPE_MVME166 },
+	{"mvme167",  VME_TYPE_MVME167 },
+	{"mvme172",  VME_TYPE_MVME172 },
+	{"mvme177",  VME_TYPE_MVME177 },
+	{"bvme4000", VME_TYPE_BVME4000},
+	{"bvme6000", VME_TYPE_BVME6000}
+};
+
+#define NUMVMETYPES (sizeof(vmetypes) / sizeof(vmetypes[0]))
+
+int
+check_arch
+(	char			*text
+)
+{	char			name[10];
+	unsigned long	vme_type;
+	int				i;
+
+	/* if no architecture specified */
+	text = skip_white(text);
+	if (*text == '\0')
+	{
+		return TRUE;
+	}
+
+	/* get VME type code for booting crate */
+	vme_type = get_vme_type();
+
+	do	{
+		/* get the next specified architecture name */
+		text = extract(name, text, sizeof(name), '\0');
+
+		/* check if it matches */
+		for (i = 0; i < NUMVMETYPES; i++)
+		{
+			if (equal_strings(vmetypes[i].name, name))
+			{
+				if (vmetypes[i].code == vme_type)
+				{
+					return TRUE;
+				}
+				break;
+			}
+		}
+		
+	} while (*text);
+
+	return FALSE;
+}
+	
+/*--------------------------------------------------------------------------*/
+
 char *
 mkinet
 (	unsigned long	ip
@@ -1662,7 +1782,7 @@ tftp
  */
 
 int
-cpu_type
+get_cpu_type
 (	void
 )
 {	int		cpu;
@@ -1698,7 +1818,7 @@ Illegal:
  */
 
 int
-fpu_type
+get_fpu_type
 (	void
 )
 {	int		fpu;
